/**
 * Copyright ProjectNya ( http://wonderfl.net/user/ProjectNya )
 * MIT License ( http://www.opensource.org/licenses/mit-license.php )
 * Downloaded from: http://wonderfl.net/c/10sP
 */

////////////////////////////////////////////////////////////////////////////////
// QuickBox2D (2)
//
// [AS3.0] QuickBox2Dを試すのだ！ (9)
// http://www.project-nya.jp/modules/weblog/details.php?blog_id=1425
////////////////////////////////////////////////////////////////////////////////

package {

    import flash.display.Sprite;
    import flash.display.MovieClip;
    import flash.geom.Rectangle;
    import flash.events.MouseEvent;
    import flash.utils.Dictionary;
    import flash.geom.Matrix;
    import flash.display.GradientType;
    import flash.display.SpreadMethod;
    import flash.display.InterpolationMethod;
    import flash.filters.BlurFilter;
    import com.actionsnippet.qbox.QuickBox2D;
    import com.actionsnippet.qbox.QuickObject;
    import Box2D.Common.Math.b2Vec2;
    import org.libspark.betweenas3.BetweenAS3;
    import org.libspark.betweenas3.tweens.ITween;
    import org.libspark.betweenas3.events.TweenEvent;
    import org.libspark.betweenas3.easing.*;

    [SWF(backgroundColor="#FFFFFF", width="465", height="465", frameRate="30")]

    public class Main extends Sprite {
        private var container:MovieClip;
        private var world:QuickBox2D;
        private var circles:Array;
        private var dictionary:Dictionary;
        private static var max:uint = 50;
        private var id:uint = 0;
        private var selectedId:uint = 0;
        private var center:QuickObject;
        private var resetBtn:Btn;

        public function Main() {
            //Wonderfl.capture_delay(1);
            init();
        }

        private function init():void {
            draw();
            //
            var label:Label = new Label(200, 24, 20, Label.CENTER);
            addChild(label);
            label.x = 132;
            label.y = 170;
            label.textColor = 0xFFFFFF;
            label.text = "QuickBox2D";
            label.alpha = 0.6;
            var txt:Label = new Label(200, 16, 16, Label.CENTER);
            addChild(txt);
            txt.x = 132;
            txt.y = 200;
            txt.textColor = 0xFFFFFF;
            txt.text = "circles";
            txt.alpha = 0.6;
            //
            container = new MovieClip();
            addChild(container);
            world = new QuickBox2D(container);
            createStageWalls(new Rectangle(0, 0, 465, 400));
            world.setDefault({lineAlpha: 0, fillColor: 0xFFFFFF, fillAlpha: 0.6, density: 0.1, friction: 1, restitution: 0.4, linearDamping: 1, angularDamping: 2, allowSleep: true});
            world.addBox({x: 2.75, y: 13.5, width: 3, height: 1, fillColor: 0x000000, fillAlpha: 0.2, density:0, isBullet: true});
            world.addBox({x: 7.75, y: 13.5, width: 2, height: 1, fillColor: 0x000000, fillAlpha: 0.2, density:0, isBullet: true});
            world.addBox({x: 12.75, y: 13.5, width: 3, height: 1, fillColor: 0x000000, fillAlpha: 0.2, density:0, isBullet: true});
            circles = new Array();
            dictionary = new Dictionary(true);
            setup();
            //
            resetBtn = new Btn();
            addChild(resetBtn);
            resetBtn.x = 232;
            resetBtn.y = 450;
            resetBtn.init({label: "reset"});
            resetBtn.addEventListener(MouseEvent.CLICK, reset, false, 0, true);
            resetBtn.enabled = false;
        }
        private function reset(evt:MouseEvent):void {
            resetBtn.enabled = false;
            world.stop();
            close();
        }
        private function setup():void {
            for (var n:uint = 0; n < max; n++) {
                var px:Number = 1.5*(n%10) + 1;
                var py:Number = 1.5*uint(n/10) + 2;
                var circle:QuickObject = world.addCircle({id: id, itween: null, selected: false, x: px, y: py, radius: 1/2, isBullet: false});
                circle.userData.buttonMode = true;
                circle.userData.mouseChildren = false;
                circle.userData.addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
                circle.userData.addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
                circle.userData.addEventListener(MouseEvent.CLICK, click, false, 0, true);
                circles.push(circle);
                dictionary[circle.userData] = circle;
                id ++;
            }
        }
        private function locate():void {
            for (var n:uint = 0; n < max; n++) {
                var circle:QuickObject = circles[n];
                var px:Number = 1.5*(n%12) + 1;
                var py:Number = 1.5*uint(n/12) + 2;
                circle.setLoc(px, py);
                circle.angle = 0;
                circle.body.SetLinearVelocity(new b2Vec2(0, 0));
                circle.body.WakeUp();
            }
            world.updateBodies();
        }
        private function rollOver(evt:MouseEvent):void {
            var circle:QuickObject = dictionary[evt.target];
            circle.body.WakeUp();
            var itween:ITween = circle.params.itween;
            if (itween) itween.stop();
            itween = circle.params.itween = BetweenAS3.parallel(
                BetweenAS3.to(circle.shape, {m_radius: 1}, 0.8, Elastic.easeOutWith(0.6, 0.3)), 
                BetweenAS3.to(circle.userData, {scaleX: 2, scaleY: 2}, 0.8, Elastic.easeOutWith(0.6, 0.3))
            );
            itween.play();
        }
        private function rollOut(evt:MouseEvent):void {
            var circle:QuickObject = dictionary[evt.target];
            if (!circle.params.selected) circle.body.WakeUp();
            var itween:ITween = circle.params.itween;
            if (itween) itween.stop();
            itween = circle.params.itween = BetweenAS3.parallel(
                BetweenAS3.to(circle.shape, {m_radius: 1/2}, 0.2, Quad.easeOut), 
                BetweenAS3.to(circle.userData, {scaleX: 1, scaleY: 1}, 0.2, Quad.easeOut)
            );
            itween.play();
        }
        private function click(evt:MouseEvent):void {
            var circle:QuickObject = dictionary[evt.target];
            var px:Number = Math.random()*13.5 + 1;
            circle.x = px;
            circle.y = -4;
            circle.angle = 0;
            circle.params.selected = true;
            circle.body.PutToSleep();
            if (!center) {
                open(circle);
            } else {
                show(circle);
            }
            world.start();
            if (!resetBtn.enabled) resetBtn.enabled = true;
        }
        private function open(circle:QuickObject):void {
            selectedId = circle.params.id;
            center = world.addCircle({itween: null, x: 7.75, y: 6.4, radius: 3, density: 0, lineAlpha: 0, fillColor: 0x000000, fillAlpha: 0.4, restitution: 0.4, isBullet: false});
            center.shape.m_radius = 0;
            center.userData.scaleX = center.userData.scaleY = 0;
            var itween:ITween = center.params.itween;
            if (itween) itween.stop();
            itween = center.params.itween = BetweenAS3.parallel(
                BetweenAS3.to(center.shape, {m_radius: 3}, 0.8, Elastic.easeOutWith(0.6, 0.5)), 
                BetweenAS3.to(center.userData, {scaleX: 1, scaleY: 1}, 0.8, Elastic.easeOutWith(0.6, 0.5))
            );
            itween.play();
        }
        private function show(circle:QuickObject):void {
            recall();
            selectedId = circle.params.id;
        }
        private function close():void {
            var itween:ITween = center.params.itween;
            if (itween) itween.stop();
            itween = center.params.itween = BetweenAS3.parallel(
                BetweenAS3.to(center.shape, {m_radius: 0}, 0.2, Expo.easeIn), 
                BetweenAS3.to(center.userData, {scaleX: 0, scaleY: 0}, 0.2, Expo.easeIn)
            );
            itween.addEventListener(TweenEvent.COMPLETE, closed, false, 0, true);
            itween.play();
        }
        private function closed(evt:TweenEvent):void {
            evt.target.removeEventListener(TweenEvent.COMPLETE, closed);
            remove();
            locate();
        }
        private function recall():void {
            var circle:QuickObject = circles[selectedId];
            circle.y = -2;
            circle.angle = 0;
            circle.params.selected = false;
            circle.body.WakeUp();
        }
        private function remove():void {
            center.fullDestroy();
            center = null;
        }
        private function createStageWalls(area:Rectangle):void {
            var px:Number = area.x/30;
            var py:Number = area.y/30;
            var sw:Number = area.width/30;
            var sh:Number = area.height/30;
            world.addBox({x: px + sw/2, y: py + sh + 0.5, width: sw, height: 1, density: 0, lineAlpha: 0, fillAlpha: 0, restitution: 0.8});
            //world.addBox({x: px + sw/2, y: py - 0.5, width: sw, height: 1, density: 0, lineAlpha: 0, fillAlpha: 0});
            world.addBox({x: px - 0.5, y: py + sh/2, width: 1, height: sh + 1, density: 0, lineAlpha: 0, fillAlpha: 0});
            world.addBox({x: px + sw + 0.5, y: py + sh/2, width: 1, height: sh + 1, density: 0, lineAlpha: 0, fillAlpha: 0});
        }
        private function draw():void {
            drawSky();
            drawGrond();
        }
        private function drawSky():void {
            var colors:Array = [0x00AAE4, 0x0069A0];
            var alphas:Array = [1, 1];
            var ratios:Array = [0, 255];
            var matrix:Matrix = new Matrix();
            matrix.createGradientBox(465, 375, Math.PI*1.5, 0, 0);
            graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
            graphics.drawRect(0, 0, 465, 375);
            graphics.endFill();
        }
        private function drawGrond():void {
            var colors:Array = [0x99CC33, 0x7EB133];
            var alphas:Array = [1, 1];
            var ratios:Array = [0, 255];
            var matrix:Matrix = new Matrix();
            matrix.createGradientBox(465, 60, 0.5*Math.PI, 0, 0);
            graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix);
            graphics.drawRect(0, 375, 465, 60);
            graphics.endFill();
        }
        
    }

}


//////////////////////////////////////////////////
// Btnクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.filters.GlowFilter;
import flash.events.MouseEvent;

class Btn extends Sprite {
    public var id:uint;
    private var shade:Shape;
    private var bottom:Shape;
    private var light:Shape;
    private var base:Shape;
    private var txt:TextField;
    private var label:String = "";
    private static var fontType:String = "_ゴシック";
    private var _width:uint = 60;
    private static var _height:uint = 20;
    private static var corner:uint = 5;
    private var type:uint = 1;
    private static var bColor:uint = 0xFFFFFF;
    private static var sColor:uint = 0x000000;
    private static var upColor:uint = 0x666666;
    private static var overColor:uint = 0x333333;
    private static var offColor:uint = 0x999999;
    private static var gColor:uint = 0x0099FF;
    private var blueGlow:GlowFilter;
    private var shadeGlow:GlowFilter;
    private var _clicked:Boolean = false;
    private var _enabled:Boolean = true;

    public function Btn() {
    }

    public function init(option:Object):void {
        if (option.id != undefined) id = option.id;
        if (option.label != undefined) label = option.label;
        if (option.width != undefined) _width = option.width;
        if (option.type != undefined) type = option.type;
        draw();
    }
    private function draw():void {
        switch (type) {
        case 1 :
            bColor = 0xFFFFFF;
            sColor = 0x000000;
            upColor = 0x666666;
            overColor = 0x333333;
            offColor = 0x999999;
            break;
        case 2 :
            bColor = 0x000000;
            sColor = 0xFFFFFF;
            upColor = 0x666666;
            overColor = 0x999999;
            offColor = 0x333333;
            break;
        }
        blueGlow = new GlowFilter(gColor, 0.6, 5, 5, 2, 3, false, true);
        shadeGlow = new GlowFilter(sColor, 0.3, 4, 4, 2, 3, false, true);
        shade = new Shape();
        bottom = new Shape();
        light = new Shape();
        base = new Shape();
        txt = new TextField();
        addChild(shade);
        addChild(bottom);
        addChild(light);
        addChild(base);
        addChild(txt);
        createBase(shade, _width, _height, corner, sColor);
        shade.filters = [shadeGlow];
        createBase(bottom, _width, _height, corner, sColor, 0.3);
        createBase(light, _width, _height, corner, gColor);
        light.filters = [blueGlow];
        createBase(base, _width, _height, corner, bColor);
        txt.x = -_width*0.5;
        txt.y = -_height*0.5;
        txt.width = _width;
        txt.height = _height - 1;
        txt.type = TextFieldType.DYNAMIC;
        txt.selectable = false;
        //txt.embedFonts = true;
        //txt.antiAliasType = AntiAliasType.ADVANCED;
        var tf:TextFormat = new TextFormat();
        tf.font = fontType;
        tf.size = 12;
        tf.align = TextFormatAlign.CENTER;
        txt.defaultTextFormat = tf;
        txt.text = label;
        enabled = true;
        mouseChildren = false;
    }
    private function rollOver(evt:MouseEvent):void {
        _over();
    }
    private function rollOut(evt:MouseEvent):void {
        _up();
    }
    private function press(evt:MouseEvent):void {
        _down();
    }
    private function release(evt:MouseEvent):void {
        _up();
    }
    private function click(evt:MouseEvent):void {
    }
    private function _up():void {
        txt.y = -_height*0.5;
        txt.textColor = upColor;
        base.y = -1;
        light.visible = false;
        light.y = -1;
    }
    private function _over():void {
        txt.y = -_height*0.5;
        txt.textColor = overColor;
        base.y = -1;
        light.visible = true;
        light.y = -1;
    }
    private function _down():void {
        txt.y = -_height*0.5 + 1;
        txt.textColor = overColor;
        base.y = 0;
        light.visible = true;
        light.y = 0;
    }
    private function _off():void {
        txt.y = -_height*0.5 + 1;
        txt.textColor = offColor;
        base.y = 0;
        light.visible = false;
        light.y = 0;
    }
    public function get clicked():Boolean {
        return _clicked;
    }
    public function set clicked(param:Boolean):void {
        _clicked = param;
        enabled = !_clicked;
        if (_clicked) {
            _down();
        } else {
            _up();
        }
    }
    public function get enabled():Boolean {
        return _enabled;
    }
    public function set enabled(param:Boolean):void {
        _enabled = param;
        buttonMode = _enabled;
        mouseEnabled = _enabled;
        useHandCursor = _enabled;
        if (_enabled) {
            _up();
            addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
            addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
            addEventListener(MouseEvent.MOUSE_DOWN, press, false, 0, true);
            addEventListener(MouseEvent.MOUSE_UP, release, false, 0, true);
            addEventListener(MouseEvent.CLICK, click, false, 0, true);
        } else {
            _off();
            removeEventListener(MouseEvent.MOUSE_OVER, rollOver);
            removeEventListener(MouseEvent.MOUSE_OUT, rollOut);
            removeEventListener(MouseEvent.MOUSE_DOWN, press);
            removeEventListener(MouseEvent.MOUSE_UP, release);
            removeEventListener(MouseEvent.CLICK, click);
        }
    }
    private function createBase(target:Shape, w:uint, h:uint, c:uint, color:uint, alpha:Number = 1):void {
        target.graphics.beginFill(color, alpha);
        target.graphics.drawRoundRect(-w*0.5, -h*0.5, w, h, c*2);
        target.graphics.endFill();
    }

}


//////////////////////////////////////////////////
// Labelクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.text.TextField;
import flash.text.TextFieldType;
import flash.text.TextFieldAutoSize;
import flash.text.AntiAliasType;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

class Label extends Sprite {
    private var txt:TextField;
    private static var fontType:String = "_ゴシック";
    private var _width:uint = 20;
    private var _height:uint = 20;
    private var size:uint = 12;
    public static const LEFT:String = TextFormatAlign.LEFT;
    public static const CENTER:String = TextFormatAlign.CENTER;
    public static const RIGHT:String = TextFormatAlign.RIGHT;

    public function Label(w:uint, h:uint, s:uint = 12, align:String = LEFT) {
        _width = w;
        _height = h;
        size = s;
        draw(align);
    }

    private function draw(align:String):void {
        txt = new TextField();
        addChild(txt);
        txt.width = _width;
        txt.height = _height;
        txt.autoSize = align;
        txt.type = TextFieldType.DYNAMIC;
        txt.selectable = false;
        //txt.embedFonts = true;
        //txt.antiAliasType = AntiAliasType.ADVANCED;
        var tf:TextFormat = new TextFormat();
        tf.font = fontType;
        tf.size = size;
        tf.align = align;
        txt.defaultTextFormat = tf;
        textColor = 0x000000;
    }
    public function set text(param:String):void {
        txt.text = param;
    }
    public function set textColor(param:uint):void {
        txt.textColor = param;
    }

}
